# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Minimal Flask application example for development.

Run example development server:

.. code-block:: console

   $ cd examples
   $ python app.py
"""

from __future__ import absolute_import, print_function

import os
from time import sleep

from flask import Flask
from flask_menu import Menu as FlaskMenu
from invenio_access import InvenioAccess
from invenio_accounts import InvenioAccounts
from invenio_accounts.testutils import create_test_user
from invenio_accounts.views import blueprint as blueprint_user
from invenio_admin import InvenioAdmin
from invenio_db import InvenioDB, db
from invenio_i18n import InvenioI18N
from invenio_indexer import InvenioIndexer
from invenio_indexer.api import RecordIndexer
from invenio_mail import InvenioMail as Mail
from invenio_pidstore import InvenioPIDStore
from invenio_records import InvenioRecords
from invenio_records_ui import InvenioRecordsUI
from invenio_search import InvenioSearch
from invenio_userprofiles import InvenioUserProfiles
from invenio_userprofiles.views import \
    blueprint_ui_init as userprofiles_blueprint_ui_init

from zenodo_accessrequests import ZenodoAccessRequests
from zenodo_accessrequests.views.requests import blueprint as request_blueprint
from zenodo_accessrequests.views.settings import \
    blueprint as settings_blueprint

# Create Flask application
app = Flask(__name__)

app.config.update(
    #  DEBUG=True,
    CELERY_ALWAYS_EAGER=True,
    CELERY_CACHE_BACKEND="memory",
    CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
    CELERY_RESULT_BACKEND="cache",
    MAIL_SUPPRESS_SEND=True,
    TESTING=True,
    SECRET_KEY='TEST',
    SQLALCHEMY_DATABASE_URI=os.environ.get(
        'SQLALCHEMY_DATABASE_URI', 'sqlite:///app.db'
    ),
    SECURITY_PASSWORD_SALT='security-password-salt',
    RECORDS_UI_ENDPOINTS=dict(
        recid=dict(
            pid_type='recid',
            route='/records/<pid_value>',
            template='invenio_records_ui/detail.html',
        ),
        recid_access_request=dict(
            pid_type='recid',
            route='/records/<pid_value>/accessrequest',
            template='zenodo_accessrequests/access_request.html',
            view_imp='zenodo_accessrequests.views.requests.access_request',
            methods=['GET', 'POST'],
        ),
        recid_access_request_email_confirm=dict(
            pid_type='recid',
            route='/records/<pid_value>/accessrequest/<token>/confirm',
            #  template='invenio_records_ui/detail.html',
            view_imp='zenodo_accessrequests.views.requests.confirm',
        ),
    )
)

InvenioDB(app)
InvenioAccounts(app)
InvenioUserProfiles(app)
InvenioRecords(app)
InvenioI18N(app)
FlaskMenu(app)
Mail(app)
InvenioRecordsUI(app)
ZenodoAccessRequests(app)
InvenioPIDStore(app)
InvenioIndexer(app)
InvenioSearch(app)
InvenioAccess(app)
InvenioAdmin(app, permission_factory=lambda x: x,
             view_class_factory=lambda x: x)

app.register_blueprint(request_blueprint)
app.register_blueprint(settings_blueprint)
app.register_blueprint(blueprint_user)
app.register_blueprint(userprofiles_blueprint_ui_init)


@app.cli.group()
def fixtures():
    """Command for working with test data."""


@fixtures.command()
def records():
    """Load test data fixture."""
    import uuid
    from invenio_records.api import Record
    from invenio_pidstore.models import PersistentIdentifier, PIDStatus

    create_test_user()

    indexer = RecordIndexer()

    # Record 1 - Live record
    with db.session.begin_nested():
        rec_uuid = uuid.uuid4()
        pid1 = PersistentIdentifier.create(
            'recid', '1', object_type='rec', object_uuid=rec_uuid,
            status=PIDStatus.REGISTERED)
        Record.create({
            'title': 'Registered',
            'description': 'This is an awesome description',
            'control_number': '1',
            'access_right': 'restricted',
            'access_conditions': 'fuu',
            'owners': [1, 2],
            'recid': 1
        }, id_=rec_uuid)
        indexer.index_by_id(pid1.object_uuid)

    db.session.commit()

    sleep(3)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Model tests."""

from __future__ import absolute_import, print_function

from zenodo_accessrequests.helpers import Ordering, QueryOrdering


def test_selected(app):
    """Test selected."""
    assert 'a' == Ordering(['a', 'b'], 'a').selected()
    assert '-b' == Ordering(['a', 'b'], '-b').selected()
    assert Ordering(['a', 'b'], 'c').selected() is None
    assert Ordering(['a', 'b'], None).selected() is None


def test_reverse(app):
    """Test reverse."""
    assert '-a' == Ordering(['a', 'b'], 'a').reverse('a')
    assert 'a' == Ordering(['a', 'b'], '-a').reverse('a')
    assert 'b' == Ordering(['a', 'b'], 'a').reverse('b')
    assert 'b' == Ordering(['a', 'b'], '-a').reverse('b')
    assert Ordering(['a', 'b'], '-a').reverse('c') is None


def test_dir(app):
    """Test direction."""
    assert 'asc' == Ordering(['a', 'b'], 'a').dir('a')
    assert 'desc' == Ordering(['a', 'b'], '-a').dir('a')
    assert Ordering(['a', 'b'], 'a').dir('b') is None
    assert Ordering(['a', 'b'], '-a').dir('b') is None


def test_is_selected(app):
    """Test selected."""
    assert Ordering(['a', 'b'], 'a').is_selected('a') is True
    assert Ordering(['a', 'b'], '-a').is_selected('a') is True
    assert Ordering(['a', 'b'], 'a').is_selected('b') is False
    assert Ordering(['a', 'b'], '-a').is_selected('b') is False


class MockQuery(object):
    """Mock Query."""

    def order_by(self, val):
        """Mock order_by."""
        return val


def test_items(app):
    """Test selected."""
    q = MockQuery()
    assert 'a' == QueryOrdering(q, ['a', 'b'], 'a').items()
    # FIXME
    # assert 'a' == QueryOrdering(q, ['a', 'b'], '-a').items().element.text
    assert q == QueryOrdering(q, ['a', 'b'], 'c').items()



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Pytest configuration."""

from __future__ import absolute_import, print_function

import os
import uuid
from datetime import datetime

import pytest
from flask import Flask
from flask_babelex import Babel
from flask_menu import Menu as FlaskMenu
from helpers import create_access_request
from invenio_access import InvenioAccess
from invenio_accounts import InvenioAccounts
from invenio_accounts.testutils import create_test_user
from invenio_accounts.views import blueprint as blueprint_user
from invenio_db import InvenioDB
from invenio_db import db as db_
from invenio_formatter import InvenioFormatter
from invenio_mail import InvenioMail as Mail
from invenio_pidstore import InvenioPIDStore
from invenio_pidstore.models import PersistentIdentifier, PIDStatus
from invenio_records import InvenioRecords
from invenio_records.api import Record
from invenio_records_ui import InvenioRecordsUI
from invenio_records_ui.views import create_blueprint_from_app
from simplekv.memory.redisstore import RedisStore
from sqlalchemy_utils.functions import create_database, database_exists, \
    drop_database

from zenodo_accessrequests import ZenodoAccessRequests
from zenodo_accessrequests.views.requests import blueprint as request_blueprint
from zenodo_accessrequests.views.settings import \
    blueprint as settings_blueprint


@pytest.yield_fixture
def app(request):
    """Flask application fixture."""
    app_ = Flask('testapp')
    app_.config.update(
        TESTING=True,
        CELERY_ALWAYS_EAGER=True,
        CELERY_CACHE_BACKEND="memory",
        CELERY_EAGER_PROPAGATES_EXCEPTIONS=True,
        CELERY_RESULT_BACKEND="cache",
        SQLALCHEMY_DATABASE_URI=os.environ.get(
            'SQLALCHEMY_DATABASE_URI', 'sqlite://'
        ),
        SQLALCHEMY_TRACK_MODIFICATIONS=False,
        SECRET_KEY='mysecret',
        SUPPORT_EMAIL='info@zenodo.org',
        WTF_CSRF_ENABLED=False,
        SERVER_NAME='test.it',
        RECORDS_UI_ENDPOINTS=dict(
            recid=dict(
                pid_type='recid',
                route='/records/<pid_value>',
                template='invenio_records_ui/detail.html',
            ),
            recid_access_request=dict(
                pid_type='recid',
                route='/records/<pid_value>/accessrequest',
                template='zenodo_accessrequests/access_request.html',
                view_imp='zenodo_accessrequests.views.requests.access_request',
                methods=['GET', 'POST'],
            ),
            recid_access_request_email_confirm=dict(
                pid_type='recid',
                route='/records/<pid_value>/accessrequest/<token>/confirm',
                #  template='invenio_records_ui/detail.html',
                view_imp='zenodo_accessrequests.views.requests.confirm',
            ),
        ),
    )

    InvenioFormatter(app_)
    Babel(app_)
    InvenioDB(app_)
    InvenioAccounts(app_)
    InvenioRecords(app_)
    FlaskMenu(app_)
    Mail(app_)
    InvenioRecordsUI(app_)
    InvenioAccess(app_)
    ZenodoAccessRequests(app_)
    InvenioPIDStore(app_)

    app_.register_blueprint(request_blueprint)
    app_.register_blueprint(settings_blueprint)
    app_.register_blueprint(blueprint_user)
    app_.register_blueprint(create_blueprint_from_app(app_))

    with app_.app_context():
        yield app_


@pytest.yield_fixture
def db(app, request):
    """Setup database."""
    if not database_exists(str(db_.engine.url)):
        create_database(str(db_.engine.url))
    db_.create_all()

    yield db_

    drop_database(str(db_.engine.url))
    # Delete sessions in kvsession store
    if hasattr(app, 'kvsession_store') and \
            isinstance(app.kvsession_store, RedisStore):
        app.kvsession_store.redis.flushall()


@pytest.fixture
def users(app, db):
    """Demo users."""
    sender = create_test_user(
        email='sender@myemail.it',
        password='sender',
        confirmed_at=datetime.utcnow(),
    )
    receiver = create_test_user(
        email='receiver@myemail.it',
        password='receiver',
    )

    users = {
        "sender": dict(id=sender.id),
        "receiver": dict(id=receiver.id),
    }

    return users


@pytest.fixture
def access_request_confirmed(app, db, users, record_example):
    """Access Request."""
    pid_value, record = record_example
    with db.session.begin_nested():
        id_req = create_access_request(pid_value, users, confirmed=True).id
    db.session.commit()
    return id_req


@pytest.fixture
def access_request_not_confirmed(app, db, users, record_example):
    """Access Request."""
    pid_value, record = record_example
    with db.session.begin_nested():
        id_req = create_access_request(
            pid_value, users, confirmed=False).id
    db.session.commit()
    return id_req


@pytest.fixture
def record_example(app, db, users):
    """Create an example of record."""
    with db.session.begin_nested():
        rec_uuid = uuid.uuid4()
        pid = PersistentIdentifier.create(
            'recid', '1', object_type='rec', object_uuid=rec_uuid,
            status=PIDStatus.REGISTERED)
        record = Record.create({
            'title': 'Registered',
            'description': 'This is an awesome description',
            'control_number': '1',
            'access_right': 'restricted',
            'access_conditions': 'fuu',
            'owners': [users['receiver']['id']],
            'recid': 1,
        }, id_=rec_uuid)
    db.session.commit()
    result = pid.pid_value, dict(record)
    return result



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.


"""Module tests."""

from __future__ import absolute_import, print_function

from flask import Flask, session, url_for
from mock import patch

from zenodo_accessrequests import ZenodoAccessRequests
from zenodo_accessrequests.models import SecretLink


def test_version():
    """Test version import."""
    from zenodo_accessrequests import __version__
    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask('testapp')
    ext = ZenodoAccessRequests(app)
    assert 'zenodo-accessrequests' in app.extensions

    app = Flask('testapp')
    ext = ZenodoAccessRequests()
    assert 'zenodo-accessrequests' not in app.extensions
    ext.init_app(app)
    assert 'zenodo-accessrequests' in app.extensions


def test_view(app, db, users, record_example):
    """Test view."""
    pid_value, record = record_example
    with app.test_client() as client:
        res = client.get(url_for(
            'invenio_records_ui.recid_access_request', pid_value=pid_value))
        assert res.status_code == 200


def test_token_are_saved_in_session(app, db, users, record_example):
    """Test view."""
    pid_value, record = record_example
    with app.test_client() as client:
        with patch.object(SecretLink,
                          'validate_token', return_value=True):
            client.get("/", query_string=dict(token='123'))
            assert session['accessrequests-secret-token'] == '123'



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.


"""Admin interface to shared links."""

from __future__ import absolute_import, print_function

from flask_admin.contrib.sqla import ModelView
from flask_babelex import gettext as _

from .models import AccessRequest, SecretLink


class AccessRequestAdmin(ModelView):
    """Access requests admin view."""

    can_create = False
    can_edit = True
    can_delete = True

    column_list = (
        'status', 'recid', 'sender_email', 'receiver', 'created', 'modified',
        'link'
    )


class SecretLinkAdmin(ModelView):
    """Secret links admin view."""

    _can_create = False
    _can_edit = True
    _can_delete = True

    acc_view_action = 'cfgaccessrequests'
    acc_edit_action = 'cfgaccessrequests'
    acc_delete_action = 'cfgaccessrequests'

    column_list = (
        'recid', 'owner', 'created', 'expires_at', 'revoked_at', 'title'
    )


accessrequest_adminview = dict(
    modelview=AccessRequestAdmin,
    model=AccessRequest,
    category=_('Shared links')
)
secretlinks_adminview = dict(
    modelview=SecretLinkAdmin,
    model=SecretLink,
    category=_('Shared links')
)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Views for creating access requests and sending access restricted files."""

from __future__ import absolute_import, print_function

from datetime import datetime

from flask import Blueprint, abort, current_app, flash, redirect, \
    render_template, request, url_for
from flask_babelex import gettext as _
from flask_login import current_user
from invenio_db import db
from werkzeug.local import LocalProxy

from ..forms import AccessRequestForm
from ..models import AccessRequest, RequestStatus
from ..tokens import EmailConfirmationSerializer

blueprint = Blueprint(
    'zenodo_accessrequests',
    __name__,
    url_prefix="/record",
    static_folder="../static",
    template_folder="../templates",
)


#
# Template filters
#


@blueprint.app_template_filter(name="is_restricted")
def is_restricted(record):
    """Template filter to check if a record is restricted."""
    return record.get('access_right') == 'restricted' and \
        record.get('access_conditions') and \
        record.get('owners', [])


@blueprint.app_template_filter()
def is_embargoed(record):
    """Template filter to check if a record is embargoed."""
    return record.get('access_right') == 'embargoed' and \
        record.get('embargo_date') and \
        record.get('embargo_date') > datetime.utcnow().date()


@blueprint.app_template_filter()
def is_removed(record):
    """Template filter to check if a record is removed."""
    return {'primary': 'SPAM'} in record.get('collections', [])

#
# Views
#


def access_request(pid, record, template, **kwargs):
    """Create an access request."""
    recid = int(pid.pid_value)
    datastore = LocalProxy(
        lambda: current_app.extensions['security'].datastore)

    # Record must be in restricted access mode.
    if record.get('access_right') != 'restricted' or \
       not record.get('access_conditions'):
        abort(404)

    # Record must have an owner and owner must still exists.
    owners = record.get('owners', [])
    record_owners = [datastore.find_user(id=owner_id) for owner_id in owners]
    if not record_owners:
        abort(404)

    sender = None
    initialdata = dict()

    # Prepare initial form data
    if current_user.is_authenticated:
        sender = current_user
        initialdata['email'] = current_user.email
        if current_user.profile:
            initialdata['full_name'] = current_user.profile.full_name

    # Normal form validation
    form = AccessRequestForm(formdata=request.form, **initialdata)

    if form.validate_on_submit():
        accreq = AccessRequest.create(
            recid=recid,
            receiver=record_owners[0],
            sender_full_name=form.data['full_name'],
            sender_email=form.data['email'],
            justification=form.data['justification'],
            sender=sender
        )
        db.session.commit()

        if accreq.status == RequestStatus.EMAIL_VALIDATION:
            flash(_(
                "Email confirmation needed: We have sent you an email to "
                "verify your address. Please check the email and follow the "
                "instructions to complete the access request."),
                category='info')
        else:
            flash(_("Access request submitted."), category='info')
        return redirect(url_for('invenio_records_ui.recid', pid_value=recid))

    return render_template(
        template,
        pid=pid,
        record=record,
        form=form,
        owners=record_owners,
    )


def confirm(pid, record, template, **kwargs):
    """Confirm email address."""
    recid = int(pid.pid_value)

    token = request.view_args['token']

    # Validate token
    data = EmailConfirmationSerializer.compat_validate_token(token)
    if data is None:
        flash(_("Invalid confirmation link."), category='danger')
        return redirect(url_for("invenio_records_ui.recid", pid_value=recid))

    # Validate request exists.
    r = AccessRequest.query.get(data['id'])
    if not r:
        abort(404)

    # Confirm email address.
    if r.status != RequestStatus.EMAIL_VALIDATION:
        abort(404)

    r.confirm_email()
    db.session.commit()
    flash(_("Email validated and access request submitted."), category='info')

    return redirect(url_for("invenio_records_ui.recid", pid_value=recid))



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Accounts settings views for pending access requests and shared links."""

from __future__ import absolute_import, print_function

import re

from flask import Blueprint, abort, flash, redirect, render_template, \
    request, url_for
from flask_babelex import gettext as _
from flask_breadcrumbs import register_breadcrumb
from flask_login import current_user, login_required
from flask_menu import register_menu
from invenio_db import db
from jinja2 import Markup, escape, evalcontextfilter

from ..forms import ApprovalForm, DeleteForm
from ..helpers import QueryOrdering
from ..models import AccessRequest, RequestStatus, SecretLink
from ..utils import get_record

blueprint = Blueprint(
    'zenodo_accessrequests_settings',
    __name__,
    url_prefix="/account/settings/sharedlinks",
    static_folder="../static",
    template_folder="../templates",
)


_paragraph_re = re.compile(r'(?:\r\n|\r|\n){2,}')


@blueprint.app_template_filter()
@evalcontextfilter
def nl2br(eval_ctx, value):
    """Template filter to convert newlines to <br>-tags."""
    result = u'\n\n'.join(u'<p>%s</p>' % p.replace('\n', '<br>\n')
                          for p in _paragraph_re.split(escape(value)))
    if eval_ctx.autoescape:
        result = Markup(result)
    return result


@blueprint.route("/", methods=['GET', 'POST'])
@login_required
@register_menu(
    blueprint, 'settings.sharedlinks',
    _('%(icon)s Shared links', icon='<i class="fa fa-share fa-fw"></i>'),
    order=9,
    active_when=lambda: request.endpoint.startswith(
        "zenodo_accessrequests_settings.")
)
@register_breadcrumb(
    blueprint, 'breadcrumbs.settings.sharedlinks', _('Shared links')
)
def index():
    """List pending access requests and shared links."""
    query = request.args.get('query', '')
    order = request.args.get('sort', '-created')
    try:
        page = int(request.args.get('page', 1))
        per_page = int(request.args.get('per_page', 20))
    except (TypeError, ValueError):
        abort(404)

    # Delete form
    form = DeleteForm(request.form)
    if form.validate_on_submit():
        link = SecretLink.query_by_owner(current_user).filter_by(
            id=form.link.data).first()
        if link.revoke():
            flash(_("Shared link revoked."), category='success')
        db.session.commit()

    # Links
    links = SecretLink.query_by_owner(current_user).filter(
        SecretLink.revoked_at.is_(None)
    )

    # Querying
    if query:
        lquery = "%{0}%".format(query)
        links = links.filter(
            SecretLink.title.like(lquery) | SecretLink.description.like(lquery)
        )

    # Ordering
    ordering = QueryOrdering(links, ['title', 'created', 'expires_at'], order)
    links = ordering.items()

    # Pending access requests
    requests = AccessRequest.query_by_receiver(current_user).filter_by(
        status=RequestStatus.PENDING).order_by('created')

    return render_template(
        "zenodo_accessrequests/settings/index.html",
        links_pagination=links.paginate(page, per_page=per_page),
        requests=requests,
        query=query,
        order=ordering,
        get_record=get_record,
        form=DeleteForm(),
    )


@blueprint.route("/accessrequest/<int:request_id>/", methods=['GET', 'POST'])
@login_required
@register_breadcrumb(
    blueprint, 'breadcrumbs.settings.sharedlinks.accessrequest',
    _('Access request')
)
def accessrequest(request_id):
    """Accept/reject access request."""
    r = AccessRequest.get_by_receiver(request_id, current_user)
    if not r or r.status != RequestStatus.PENDING:
        abort(404)

    form = ApprovalForm(request.form)

    if form.validate_on_submit():
        if form.accept.data:
            r.accept(message=form.data['message'],
                     expires_at=form.expires_at.data)
            db.session.commit()
            flash(_("Request accepted."))
            return redirect(url_for(".index"))
        elif form.reject.data:
            r.reject(message=form.data['message'])
            db.session.commit()
            flash(_("Request rejected."))
            return redirect(url_for(".index"))

    pid, record = get_record(r.recid)
    return render_template(
        "zenodo_accessrequests/settings/request.html",
        accessrequest=r,
        record=record,
        form=form,
    )



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Token serializers."""

from __future__ import absolute_import, print_function

import binascii
import os
from base64 import urlsafe_b64encode
from datetime import datetime

from flask import current_app
from itsdangerous import BadData, JSONWebSignatureSerializer, \
    SignatureExpired, TimedJSONWebSignatureSerializer

SUPPORTED_DIGEST_ALGORITHMS = ('HS256', 'HS512')


class TokenMixin(object):
    """Mix-in class for token serializers."""

    def create_token(self, obj_id, extra_data):
        """Create a token referencing the object id with extra data.

        Note random data is added to ensure that no two tokens are identical.
        """
        return self.dumps(
            dict(
                id=obj_id,
                data=extra_data,
                rnd=binascii.hexlify(os.urandom(4)).decode('utf-8')
            )
        )

    def validate_token(self, token, expected_data=None):
        """Validate secret link token.

        :param token: Token value.
        :param expected_data: A dictionary of key/values that must be present
            in the data part of the token (i.e. included via ``extra_data`` in
            ``create_token``).
        """
        try:
            # Load token and remove random data.
            data = self.load_token(token)

            # Compare expected data with data in token.
            if expected_data:
                for k in expected_data:
                    if expected_data[k] != data["data"].get(k):
                        return None
            return data
        except BadData:
            return None

    def load_token(self, token, force=False):
        """Load data in a token.

        :param token: Token to load.
        :param force: Load token data even if signature expired.
                      Default: False.
        """
        try:
            data = self.loads(token)
        except SignatureExpired as e:
            if not force:
                raise
            data = e.payload

        del data["rnd"]
        return data


class EncryptedTokenMixIn(TokenMixin):
    """Mix-in class for token serializers that generate encrypted tokens."""

    @property
    def engine(self):
        """Get cryptographic engine."""
        if not hasattr(self, '_engine'):
            from cryptography.fernet import Fernet
            from cryptography.hazmat.backends import default_backend
            from cryptography.hazmat.primitives import hashes

            digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
            digest.update(current_app.config['SECRET_KEY'].encode('utf8'))
            fernet_key = urlsafe_b64encode(digest.finalize())
            self._engine = Fernet(fernet_key)
        return self._engine

    def create_token(self, obj_id, extra_data):
        """Create a token referencing the object id with extra data."""
        return self.engine.encrypt(
            super(EncryptedTokenMixIn, self).create_token(obj_id, extra_data)
        )

    def load_token(self, token, force=False):
        """Load data in a token.

        :param token: Token to load.
        :param force: Load token data even if signature expired.
                      Default: False.
        """
        return super(EncryptedTokenMixIn, self).load_token(
            self.engine.decrypt(token), force=force
        )


class EmailConfirmationSerializer(TimedJSONWebSignatureSerializer, TokenMixin):
    """Serializer for email confirmation link tokens.

    Depends upon the secrecy of ``SECRET_KEY``. Tokens expire after a specific
    time (defaults to ``ACCESSREQUESTS_CONFIRMLINK_EXPIRES_IN``). The
    access request id as well as the email address is stored in the token
    together with a random bit to ensure all tokens are unique.
    """

    def __init__(self, expires_in=None, **kwargs):
        """Initialize underlying TimedJSONWebSignatureSerializer."""
        dt = expires_in or \
            current_app.config['ACCESSREQUESTS_CONFIRMLINK_EXPIRES_IN']

        super(EmailConfirmationSerializer, self).__init__(
            current_app.config['SECRET_KEY'],
            expires_in=dt,
            salt='accessrequests-email',
            **kwargs
        )

    @classmethod
    def compat_validate_token(cls, *args, **kwargs):
        """Multiple algorithm-compatible token validation."""
        for algorithm in SUPPORTED_DIGEST_ALGORITHMS:
            try:
                cls(algorithm_name=algorithm).validate_token(*args, **kwargs)
            except BadData:  # move to next algorithm
                continue


class SecretLinkSerializer(JSONWebSignatureSerializer, TokenMixin):
    """Serializer for secret links."""

    def __init__(self, **kwargs):
        """Initialize underlying JSONWebSignatureSerializer."""
        super(SecretLinkSerializer, self).__init__(
            current_app.config['SECRET_KEY'],
            salt='accessrequests-link',
            **kwargs
        )


class TimedSecretLinkSerializer(TimedJSONWebSignatureSerializer,
                                TokenMixin):
    """Serializer for expiring secret links."""

    def __init__(self, expires_at=None, **kwargs):
        """Initialize underlying TimedJSONWebSignatureSerializer."""
        assert isinstance(expires_at, datetime) or expires_at is None

        dt = expires_at - datetime.now() if expires_at else None

        super(TimedSecretLinkSerializer, self).__init__(
            current_app.config['SECRET_KEY'],
            expires_in=int(dt.total_seconds()) if dt else None,
            salt='accessrequests-timedlink',
            **kwargs
        )


class SecretLinkFactory(object):
    """Functions for creating and validating any secret link tokens."""

    @classmethod
    def create_token(cls, obj_id, data, expires_at=None):
        """Create the secret link token."""
        if expires_at:
            s = TimedSecretLinkSerializer(expires_at=expires_at)
        else:
            s = SecretLinkSerializer()

        return s.create_token(obj_id, data)

    @classmethod
    def validate_token(cls, token, expected_data=None):
        """Validate a secret link token (non-expiring + expiring)."""
        for algorithm in SUPPORTED_DIGEST_ALGORITHMS:
            s = SecretLinkSerializer(algorithm_name=algorithm)
            st = TimedSecretLinkSerializer(algorithm_name=algorithm)

            try:
                for serializer in (s, st):
                    data = serializer.validate_token(
                        token, expected_data=expected_data)
                    if data:
                        return data
            except SignatureExpired:  # move to next algorithm
                raise
            except BadData:
                continue  # move to next serializer/algorithm

    @classmethod
    def load_token(cls, token, force=False):
        """Validate a secret link token (non-expiring + expiring)."""
        for algorithm in SUPPORTED_DIGEST_ALGORITHMS:
            s = SecretLinkSerializer(algorithm_name=algorithm)
            st = TimedSecretLinkSerializer(algorithm_name=algorithm)
            for serializer in (s, st):
                try:
                    data = serializer.load_token(token, force=force)
                    if data:
                        return data
                except SignatureExpired:
                    raise  # signature was parsed and is expired
                except BadData:
                    continue  # move to next serializer/algorithm



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Default configuration values."""

from __future__ import absolute_import, print_function

ACCESSREQUESTS_CONFIRMLINK_EXPIRES_IN = 5*24*60*60
"""Number of seconds after the email confirmation link expires."""

ACCESSREQUESTS_RECORDS_UI_ENDPOINTS = dict(
    recid_access_request=dict(
        pid_type='recid',
        route='/record/<pid_value>/accessrequest',
        template='zenodo_accessrequests/access_request.html',
        view_imp='zenodo_accessrequests.views.requests.access_request',
        methods=['GET', 'POST'],
    ),
    recid_access_request_email_confirm=dict(
        pid_type='recid',
        route='/record/<pid_value>/accessrequest/<token>/confirm',
        view_imp='zenodo_accessrequests.views.requests.confirm',
    ),
)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Token creation and validation test case."""

from __future__ import absolute_import, print_function

from datetime import datetime, timedelta

import pytest
from itsdangerous import BadData, BadSignature, JSONWebSignatureSerializer, \
    SignatureExpired

from zenodo_accessrequests.tokens import EmailConfirmationSerializer, \
    EncryptedTokenMixIn, SecretLinkFactory, SecretLinkSerializer, \
    TimedSecretLinkSerializer


def test_email_confirmation_serializer_create_validate(app, db):
    """Test token creation."""
    extra_data = dict(email="info@invenio-software.org")
    with app.app_context():
        s = EmailConfirmationSerializer()
        t = s.create_token(1, extra_data)
        data = s.validate_token(t, expected_data=extra_data)
        assert data['id'] == 1
        assert data['data'] == extra_data


def test_email_confirmation_serializer_expired(app, db):
    """Test token expiry."""
    extra_data = dict(email="info@invenio-software.org")
    with app.app_context():
        s = EmailConfirmationSerializer(expires_in=-20)
        t = s.create_token(1, extra_data)
        assert s.validate_token(t) is None
        assert s.validate_token(t, expected_data=extra_data) is None
        with pytest.raises(SignatureExpired):
            s.load_token(t)
        assert s.load_token(t, force=True) is not None


def test_email_confirmation_serializer_expected_data_mismatch(app, db):
    """Test token validation."""
    extra_data = dict(email="info@invenio-software.org")
    with app.app_context():
        s = EmailConfirmationSerializer()
        t = s.create_token(1, extra_data)
        assert s.validate_token(t) is not None
        assert s.validate_token(t, dict(notvalid=1)) is None
        assert s.validate_token(t, dict(email='another@email')) is None


def test_email_confirmation_serializer_creation(app, db):
    """Ensure that no two tokens are identical."""
    extra_data = dict(email="info@invenio-software.org")
    with app.app_context():
        s = EmailConfirmationSerializer()
        t1 = s.create_token(1, extra_data)
        t2 = s.create_token(1, extra_data)
        assert t1 != t2


def test_secretlink_serializer_create_validate(app, db):
    """Test token creation."""
    with app.app_context():
        s = SecretLinkSerializer()
        t = s.create_token(1234, dict(recid=56789))
        data = s.validate_token(t)
        assert data['id'] == 1234
        assert data['data']['recid'] == 56789


def test_secretlink_serializer_creation(app, db):
    """Ensure that no two tokens are identical."""
    with app.app_context():
        s = SecretLinkSerializer()
        t1 = s.create_token(98765, dict(recid=4321))
        t2 = s.create_token(98765, dict(recid=4321))
        assert t1 != t2


def test_secretlink_serializer_noencryption(app, db):
    """Test that token is not encrypted."""
    with app.app_context():
        s = SecretLinkSerializer()
        t1 = s.create_token(1, dict(recid=1))
        with pytest.raises(BadSignature):
            JSONWebSignatureSerializer('anotherkey').loads(t1)


class TestSerializer(EncryptedTokenMixIn, SecretLinkSerializer):
    """Test serializer."""
    pass


def test_encryptedtoken_mixin_create_validate(app, db):
    """Test token creation."""
    with app.app_context():
        s = TestSerializer()
        t = s.create_token(1234, dict(recid=56789))
        data = s.validate_token(t)
        assert data['id'] == 1234
        assert data['data']['recid'] == 56789


def test_encryptedtoken_mixin_creation(app, db):
    """Ensure that no two tokens are identical."""
    with app.app_context():
        s = TestSerializer()
        t1 = s.create_token(98765, dict(recid=4321))
        t2 = s.create_token(98765, dict(recid=4321))
        assert t1 != t2


def test_encryptedtoken_mixin_encryption(app, db):
    """Test that token is not encrypted."""
    with app.app_context():
        s = TestSerializer()
        t1 = s.create_token(1, dict(recid=1))
        with pytest.raises(BadData):
            JSONWebSignatureSerializer('anotherkey').loads(t1)


def test_timed_secretlink_serializer_create_validate(app, db):
    """Test token creation."""
    with app.app_context():
        s = TimedSecretLinkSerializer(
            expires_at=datetime.utcnow()+timedelta(days=1))
        t = s.create_token(1234, dict(recid=56789))
        data = s.validate_token(t, expected_data=dict(recid=56789))
        assert data['id'] == 1234
        assert data['data']['recid'] == 56789
        assert s.validate_token(t, expected_data=dict(recid=1)) is None


def test_timed_secretlink_serializer_expired(app, db):
    """Test token expiry."""
    with app.app_context():
        s = TimedSecretLinkSerializer(
            expires_at=datetime.utcnow()-timedelta(seconds=20))
        t = s.create_token(1, dict(recid=1))
        assert s.validate_token(t) is None
        assert s.validate_token(t, expected_data=dict(recid=1)) is None
        with pytest.raises(SignatureExpired):
            s.load_token(t)
        assert s.load_token(t, force=True) is not None


def test_timed_secretlink_serializer_creation(app, db):
    """Ensure that no two tokens are identical."""
    with app.app_context():
        s = TimedSecretLinkSerializer(
            expires_at=datetime.utcnow()+timedelta(days=1))
        t1 = s.create_token(98765, dict(recid=4321))
        t2 = s.create_token(98765, dict(recid=4321))
        assert t1 != t2


def test_secretlink_factory_validation(app, db):
    """Test token validation."""
    extra_data = dict(recid='1')
    with app.app_context():
        t = SecretLinkFactory.create_token(1, extra_data)
        assert SecretLinkFactory.validate_token(
            t, expected_data=extra_data) is not None

        t = SecretLinkFactory.create_token(
            1, extra_data, expires_at=datetime.utcnow()+timedelta(days=1)
        )
        assert SecretLinkFactory.validate_token(
            t, expected_data=extra_data) is not None
        assert SecretLinkFactory.validate_token(
            t, expected_data=dict(recid=2)) is None


def test_secretlink_factory_creation(app, db):
    """Test token creation."""
    extra_data = dict(recid='1')
    with app.app_context():
        d = datetime.utcnow()+timedelta(days=1)

        t = SecretLinkFactory.create_token(1, extra_data)

        assert SecretLinkSerializer().validate_token(
            t, expected_data=extra_data) is not None
        assert TimedSecretLinkSerializer().validate_token(
            t, expected_data=extra_data) is None

        t1 = SecretLinkFactory.create_token(
            1, extra_data, expires_at=d
        )
        t2 = SecretLinkFactory.create_token(1, extra_data)

        assert SecretLinkSerializer().validate_token(
            t1, expected_data=extra_data) is None
        assert TimedSecretLinkSerializer().validate_token(
            t1, expected_data=extra_data) is not None
        assert t1 != t2


def test_secretlink_factory_load_token(app, db):
    """Test token loading."""
    extra_data = dict(recid='1')
    with app.app_context():
        t = SecretLinkFactory.create_token(1, extra_data)
        assert SecretLinkFactory.load_token(t) is not None

        t = SecretLinkFactory.create_token(
            1, extra_data, expires_at=datetime.utcnow()-timedelta(days=1))
        with pytest.raises(SignatureExpired):
            SecretLinkFactory.load_token(t)
        assert SecretLinkFactory.load_token(t, force=True) is not None



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Signal receivers."""

from __future__ import absolute_import, print_function

from datetime import timedelta

from flask import current_app, render_template, url_for
from flask_babelex import gettext as _
from flask_mail import Message
from invenio_mail.tasks import send_email

from .errors import RecordNotFound
from .signals import request_accepted, request_confirmed, request_created, \
    request_rejected
from .tokens import EmailConfirmationSerializer
from .utils import get_record


def connect_receivers():
    """Connect receivers to signals."""
    request_created.connect(send_email_validation)
    request_confirmed.connect(send_confirmed_notifications)
    request_rejected.connect(send_reject_notification)
    # Order is important:
    request_accepted.connect(create_secret_link)
    request_accepted.connect(send_accept_notification)


def create_secret_link(request, message=None, expires_at=None):
    """Receiver for request-accepted signal."""
    pid, record = get_record(request.recid)
    if not record:
        raise RecordNotFound(request.recid)

    description = render_template(
        "zenodo_accessrequests/link_description.tpl",
        request=request,
        record=record,
        pid=pid,
        expires_at=expires_at,
        message=message,
    )

    request.create_secret_link(
        record["title"],
        description=description,
        expires_at=expires_at
    )


def send_accept_notification(request, message=None, expires_at=None):
    """Receiver for request-accepted signal to send email notification."""
    pid, record = get_record(request.recid)
    _send_notification(
        request.sender_email,
        _("Access request accepted"),
        "zenodo_accessrequests/emails/accepted.tpl",
        request=request,
        record=record,
        pid=pid,
        record_link=request.link.get_absolute_url('invenio_records_ui.recid'),
        message=message,
        expires_at=expires_at,
    )


def send_confirmed_notifications(request):
    """Receiver for request-confirmed signal to send email notification."""
    pid, record = get_record(request.recid)
    if record is None:
        current_app.logger.error("Cannot retrieve record %s. Emails not sent"
                                 % request.recid)
        return
    title = _("Access request: %(record)s", record=record["title"])

    _send_notification(
        request.receiver.email,
        title,
        "zenodo_accessrequests/emails/new_request.tpl",
        request=request,
        record=record,
        pid=pid,
    )

    _send_notification(
        request.sender_email,
        title,
        "zenodo_accessrequests/emails/confirmation.tpl",
        request=request,
        record=record,
        pid=pid,
    )


def send_email_validation(request):
    """Receiver for request-created signal to send email notification."""
    token = EmailConfirmationSerializer().create_token(
        request.id, dict(email=request.sender_email)
    )
    pid, record = get_record(request.recid)

    _send_notification(
        request.sender_email,
        _("Access request verification"),
        "zenodo_accessrequests/emails/validate_email.tpl",
        request=request,
        record=record,
        pid=pid,
        days=timedelta(
            seconds=current_app.config["ACCESSREQUESTS_CONFIRMLINK_EXPIRES_IN"]
        ).days,
        confirm_link=url_for(
            "invenio_records_ui.recid_access_request_email_confirm",
            pid_value=request.recid,
            token=token,
            _external=True,
        )
    )


def send_reject_notification(request, message=None):
    """Receiver for request-rejected signal to send email notification."""
    pid, record = get_record(request.recid)
    _send_notification(
        request.sender_email,
        _("Access request rejected"),
        "zenodo_accessrequests/emails/rejected.tpl",
        request=request,
        record=record,
        pid=pid,
        message=message,
    )


def _send_notification(to, subject, template, **ctx):
    """Render a template and send as email."""
    msg = Message(
        subject,
        sender=current_app.config.get('SUPPORT_EMAIL'),
        recipients=[to]
    )
    msg.body = render_template(template, **ctx)

    send_email.delay(msg.__dict__)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Zenodo module for providing access request feature."""

from __future__ import absolute_import, print_function

from flask import request, session

from . import config


def verify_token():
    """Verify token and save in session if it's valid."""
    try:
        from .models import SecretLink
        token = request.args['token']
        # if the token is valid
        if token and SecretLink.validate_token(token, {}):
            # then save in session the token
            session['accessrequests-secret-token'] = token
    except KeyError:
        pass


class _AppState(object):

    def __init__(self, app):
        """Initialize state."""
        from .receivers import connect_receivers
        self.app = app
        connect_receivers()


class ZenodoAccessRequests(object):
    """Zenodo-AccessRequests extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        app.before_request(verify_token)
        self.init_config(app)
        state = _AppState(app=app)
        app.extensions['zenodo-accessrequests'] = state

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault(
            "ACCESSREQUESTS_BASE_TEMPLATE",
            app.config.get("BASE_TEMPLATE",
                           "zenodo_accessrequests/base.html"))
        app.config.setdefault(
            "ACCESSREQUESTS_SETTINGS_TEMPLATE",
            app.config.get("SETTINGS_TEMPLATE",
                           "zenodo_accessrequests/settings/base.html"))

        for k in dir(config):
            if k.startswith('ACCESSREQUESTS_'):
                app.config.setdefault(k, getattr(config, k))



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Access request signals."""

from __future__ import absolute_import, print_function

from blinker import Namespace

_signals = Namespace()

request_accepted = _signals.signal('request-accepted')

request_rejected = _signals.signal('request-rejected')

request_created = _signals.signal('request-created')

request_confirmed = _signals.signal('request-confirmed')

link_created = _signals.signal('link-created')

link_revoked = _signals.signal('link-revoked')



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Database models for shared links."""

from __future__ import absolute_import, print_function

from copy import deepcopy
from datetime import date, datetime

from flask import current_app, url_for
from flask_babelex import gettext as _
from invenio_accounts.models import User
from invenio_db import db
from sqlalchemy_utils.types import ChoiceType, EncryptedType

from .errors import InvalidRequestStateError
from .signals import link_created, link_revoked, request_accepted, \
    request_confirmed, request_created, request_rejected
from .tokens import SecretLinkFactory

# TODO: UTC timestamps + localization.


def secret_key():
    """Return secret key as bytearray."""
    return current_app.config['SECRET_KEY'].encode('utf-8')


class RequestStatus(object):
    """Access request status representation."""

    EMAIL_VALIDATION = u'C'
    PENDING = u'P'
    ACCEPTED = u'A'
    REJECTED = u'R'


class SecretLink(db.Model):
    """Represent a secret link to a record restricted files."""

    __tablename__ = 'accessrequests_link'

    id = db.Column(db.Integer, primary_key=True,
                   autoincrement=True)
    """Secret link id."""

    token = db.Column(
        EncryptedType(type_in=db.Text, key=secret_key),
        nullable=False
    )
    """Secret token for link (should be stored encrypted)."""

    owner_user_id = db.Column(
        db.Integer, db.ForeignKey(User.id),
        nullable=False, default=None
    )
    """Owner's user id."""

    owner = db.relationship(User, foreign_keys=[owner_user_id])
    """Relationship to user"""

    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow,
                        index=True)
    """Creation timestamp."""

    expires_at = db.Column(db.DateTime, nullable=True)
    """Expiration date."""

    revoked_at = db.Column(db.DateTime, nullable=True, index=True)
    """Creation timestamp."""

    title = db.Column(db.String(length=255), nullable=False, default='')
    """Title of link."""

    description = db.Column(db.Text, nullable=False, default='')
    """Description of link."""

    @classmethod
    def create(cls, title, owner, extra_data, description="", expires_at=None):
        """Create a new secret link."""
        if isinstance(expires_at, date):
            expires_at = datetime.combine(expires_at, datetime.min.time())

        with db.session.begin_nested():
            obj = cls(
                owner=owner,
                title=title,
                description=description,
                expires_at=expires_at,
                token='',
            )
            db.session.add(obj)

        with db.session.begin_nested():
            # Create token (dependent on obj.id and recid)
            obj.token = SecretLinkFactory.create_token(
                obj.id, extra_data, expires_at=expires_at
            ).decode('utf8')

        link_created.send(obj)
        return obj

    @classmethod
    def validate_token(cls, token, expected_data):
        """Validate a secret link token.

        Only queries the database if token is valid to determine that the token
        has not been revoked.
        """
        data = SecretLinkFactory.validate_token(
            token, expected_data=expected_data
        )

        if data:
            link = cls.query.get(data['id'])
            if link and link.is_valid():
                return True
        return False

    @classmethod
    def query_by_owner(cls, user):
        """Get secret links by user."""
        return cls.query.filter_by(
            owner_user_id=user.id
        )

    @property
    def extra_data(self):
        """Load token data stored in token (ignores expiry date of tokens)."""
        if self.token:
            return SecretLinkFactory.load_token(self.token, force=True)["data"]
        return None

    def get_absolute_url(self, endpoint):
        """Get absolute for secret link (using https scheme).

        The endpoint is passed to ``url_for`` with ``token`` and ``extra_data``
        as keyword arguments. E.g.::

            >>> link.extra_data
            dict(recid=1)
            >>> link.get_absolute_url('record.metadata')

        translates into::

            >>> url_for('record.metadata', token="...", recid=1, )
        """
        copy = deepcopy(self.extra_data)
        if 'recid' in copy:
            copy['pid_value'] = copy.pop('recid')
        return url_for(
            endpoint, token=self.token,
            _external=True, **(copy or {})
        )

    def revoke(self):
        """Revoken a secret link."""
        if self.revoked_at is None:
            with db.session.begin_nested():
                self.revoked_at = datetime.utcnow()
            link_revoked.send(self)
            return True
        return False

    def is_expired(self):
        """Determine if link is expired."""
        if self.expires_at:
            return datetime.utcnow() > self.expires_at
        return False

    def is_revoked(self):
        """Determine if link is revoked."""
        return self.revoked_at is not None

    def is_valid(self):
        """Determine if link is still valid."""
        return not(self.is_expired() or self.is_revoked())


class AccessRequest(db.Model):
    """Represent an request for access to restricted files in a record."""

    __tablename__ = 'accessrequests_request'

    STATUS_CODES = {
        RequestStatus.EMAIL_VALIDATION: _(u'Email validation'),
        RequestStatus.PENDING: _(u'Pending'),
        RequestStatus.ACCEPTED: _(u'Accepted'),
        RequestStatus.REJECTED: _(u'Rejected'),
    }

    id = db.Column(db.Integer, primary_key=True,
                   autoincrement=True)
    """Access request ID."""

    status = db.Column(
        ChoiceType(STATUS_CODES.items(), impl=db.CHAR(1)),
        nullable=False, index=True
    )
    """Status of request."""

    receiver_user_id = db.Column(
        db.Integer, db.ForeignKey(User.id),
        nullable=False, default=None
    )
    """Receiver's user id."""

    receiver = db.relationship(User, foreign_keys=[receiver_user_id])
    """Relationship to user"""

    sender_user_id = db.Column(
        db.Integer, db.ForeignKey(User.id),
        nullable=True, default=None
    )
    """Sender's user id (for authenticated users)."""

    sender = db.relationship(User, foreign_keys=[sender_user_id])
    """Relationship to user for a sender"""

    sender_full_name = db.Column(db.String(length=255), nullable=False,
                                 default='')
    """Sender's full name."""

    sender_email = db.Column(db.String(length=255), nullable=False,
                             default='')
    """Sender's email address."""

    recid = db.Column(db.Integer, nullable=False, index=True)
    """Record concerned for the request."""

    created = db.Column(db.DateTime, nullable=False, default=datetime.utcnow,
                        index=True)
    """Creation timestamp."""

    modified = db.Column(db.DateTime, nullable=False, default=datetime.utcnow,
                         onupdate=datetime.utcnow)
    """Last modification timestamp."""

    justification = db.Column(db.Text, default='', nullable=False)
    """Sender's justification for how they fulfill conditions."""

    message = db.Column(db.Text, default='', nullable=False)
    """Receivers message to the sender."""

    link_id = db.Column(
        db.Integer, db.ForeignKey(SecretLink.id),
        nullable=True, default=None
    )
    """Relation to secret link if request was accepted."""

    link = db.relationship(SecretLink, foreign_keys=[link_id])
    """Relationship to secret link."""

    @classmethod
    def create(cls, recid=None, receiver=None, sender_full_name=None,
               sender_email=None, justification=None, sender=None):
        """Create a new access request.

        :param recid: Record id (required).
        :param receiver: User object of receiver (required).
        :param sender_full_name: Full name of sender (required).
        :param sender_email: Email address of sender (required).
        :param justification: Justification message (required).
        :param sender: User object of sender (optional).
        """
        sender_user_id = None if sender is None else sender.id

        assert recid
        assert receiver
        assert sender_full_name
        assert sender_email
        assert justification

        # Determine status
        status = RequestStatus.EMAIL_VALIDATION
        if sender and sender.confirmed_at:
            status = RequestStatus.PENDING

        with db.session.begin_nested():
            # Create object
            obj = cls(
                status=status,
                recid=recid,
                receiver_user_id=receiver.id,
                sender_user_id=sender_user_id,
                sender_full_name=sender_full_name,
                sender_email=sender_email,
                justification=justification
            )

            db.session.add(obj)

        # Send signal
        if obj.status == RequestStatus.EMAIL_VALIDATION:
            request_created.send(obj)
        else:
            request_confirmed.send(obj)
        return obj

    @classmethod
    def query_by_receiver(cls, user):
        """Get access requests for a specific receiver."""
        return cls.query.filter_by(
            receiver_user_id=user.id
        )

    @classmethod
    def get_by_receiver(cls, request_id, user):
        """Get access request for a specific receiver."""
        return cls.query.filter_by(
            id=request_id,
            receiver_user_id=user.id
        ).first()

    def confirm_email(self):
        """Confirm that senders email is valid."""
        with db.session.begin_nested():
            if self.status != RequestStatus.EMAIL_VALIDATION:
                raise InvalidRequestStateError(RequestStatus.EMAIL_VALIDATION)

            self.status = RequestStatus.PENDING
        request_confirmed.send(self)

    def accept(self, message=None, expires_at=None):
        """Accept request."""
        with db.session.begin_nested():
            if self.status != RequestStatus.PENDING:
                raise InvalidRequestStateError(RequestStatus.PENDING)
            self.status = RequestStatus.ACCEPTED
        request_accepted.send(self, message=message, expires_at=expires_at)

    def reject(self, message=None):
        """Reject request."""
        with db.session.begin_nested():
            if self.status != RequestStatus.PENDING:
                raise InvalidRequestStateError(RequestStatus.PENDING)
            self.status = RequestStatus.REJECTED
        request_rejected.send(self, message=message)

    def create_secret_link(self, title, description=None, expires_at=None):
        """Create a secret link from request."""
        self.link = SecretLink.create(
            title,
            self.receiver,
            extra_data=dict(recid=self.recid),
            description=description,
            expires_at=expires_at,
        )
        return self.link



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Helper classes to make working with column ordering in templates easier."""

from __future__ import absolute_import, print_function

from sqlalchemy.sql.expression import desc


class Ordering(object):
    """Helper class for column sorting."""

    def __init__(self, options, selected):
        """Initialize ordering with possible options the selected option.

        :param options: List of column options.
        :param selected: Selected column. Prefix name with ``-`` to denote
            descending ordering.
        """
        self.options = options
        if selected in options:
            self._selected = selected
            self.asc = True
        elif selected and selected[0] == '-' and selected[1:] in options:
            self._selected = selected[1:]
            self.asc = False
        else:
            self._selected = None
            self.asc = None

    def reverse(self, col):
        """Get reverse direction of ordering."""
        if col in self.options:
            if self.is_selected(col):
                return col if not self.asc else '-{0}'.format(col)
            else:
                return col
        return None

    def dir(self, col, asc='asc', desc='desc'):
        """Get direction (ascending/descending) of ordering."""
        if col == self._selected and self.asc is not None:
            return asc if self.asc else desc
        else:
            return None

    def is_selected(self, col):
        """Determine if column is being order by."""
        return col == self._selected

    def selected(self):
        """Get column which is being order by."""
        if self._selected:
            return self._selected if self.asc else \
                "-{0}".format(self._selected)
        return None


class QueryOrdering(Ordering):
    """Helper class for column sorting based on SQLAlchemy queries."""

    def __init__(self, query, options, selected):
        """Initialize with SQLAlchemy query."""
        super(QueryOrdering, self).__init__(options, selected)
        self.query = query

    def items(self):
        """Get query with correct ordering."""
        if self.asc is not None:
            if self._selected and self.asc:
                return self.query.order_by(self._selected)
            elif self._selected and not self.asc:
                return self.query.order_by(desc(self._selected))
        return self.query



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Model tests."""

from __future__ import absolute_import, print_function

from datetime import datetime, timedelta

from zenodo_accessrequests.forms import AccessRequestForm, ApprovalForm


def test_validation_form_accessrequest(app):
    """Test validation."""
    assert not AccessRequestForm(
        full_name="  ", email="   ", justification="  ").validate()
    assert not AccessRequestForm(
        full_name="Test", email="", justification="").validate()
    assert not AccessRequestForm(
        full_name="Test", email="info", justification="").validate()
    assert not AccessRequestForm(
        full_name="Test", email="info", justification="info").validate()
    assert AccessRequestForm(
        full_name="Test", email="info@invenio-software.org",
        justification="info").validate()


def test_validation_form_approval(app):
    """Test validation."""
    today = datetime.utcnow().date()
    yesterday = datetime.utcnow().date()+timedelta(days=-1)
    tomorrow = datetime.utcnow().date()+timedelta(days=1)
    oneyear = datetime.utcnow().date()+timedelta(days=365)
    oneyearoneday = datetime.utcnow().date()+timedelta(days=366)

    assert not ApprovalForm(
        message="", expires_at="", accept=True).validate()
    assert not ApprovalForm(
        message="", expires_at="", reject=True).validate()
    assert not ApprovalForm(
        message="test", expires_at=tomorrow, accept=True, reject=True
    ).validate()

    # Accept + date validation
    assert ApprovalForm(
        message="", expires_at=tomorrow, accept=True).validate()
    assert ApprovalForm(
        message="", expires_at=oneyear, accept=True).validate()
    assert not ApprovalForm(
        message="", expires_at=today, accept=True).validate()
    assert not ApprovalForm(
        message="", expires_at=yesterday, accept=True).validate()
    assert not ApprovalForm(
        message="", expires_at=oneyearoneday, accept=True).validate()

    # Reject + message
    assert ApprovalForm(
        message="a message", expires_at="", reject=True).validate()
    assert not ApprovalForm(
        message="", expires_at="", reject=True).validate()
    assert not ApprovalForm(
        message="      ", expires_at="", reject=True).validate()



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.


"""Forms for module."""

from __future__ import absolute_import, print_function

from datetime import datetime, timedelta

from flask_babelex import gettext as _
from flask_security.forms import email_required, email_validator
from flask_wtf import Form
from wtforms import DateField, HiddenField, StringField, SubmitField, \
    TextAreaField, validators

from .widgets import Button


def validate_expires_at(form, field):
    """Validate that date is in the future."""
    if form.accept.data:
        if not field.data or datetime.utcnow().date() >= field.data:
            raise validators.StopValidation(_(
                "Please provide a future date."
            ))
        if not field.data or \
                datetime.utcnow().date() + timedelta(days=365) < field.data:
            raise validators.StopValidation(_(
                "Please provide a date no more than 1 year into the future."
            ))


class AccessRequestForm(Form):
    """Form for requesting access to a record."""

    full_name = StringField(
        label=_("Full name"),
        description=_("Required."),
        validators=[validators.DataRequired()]
    )

    email = StringField(
        label=_("Email address"),
        description=_(
            "Required. Please carefully check your email address. If the owner"
            " grants access, a secret link will be sent to this email address."
        ),
        validators=[email_required, email_validator]
    )

    justification = TextAreaField(
        label=_("Justification"),
        description=_(
            "Required. Please thoroughly justify how you fulfil the "
            "conditions listed above."),
        validators=[validators.DataRequired()],
    )


class ApprovalForm(Form):
    """Form used to approve/reject requests."""

    request = HiddenField()

    message = TextAreaField(
        label=_("Message to requester"),
        description=_(
            "Required if you reject the request. Optional if you accept the"
            " request."),
    )
    expires_at = DateField(
        label=_('Expires'),
        description=_(
            'Format: YYYY-MM-DD. Required if you accept the request. The '
            'access will automatically be revoked on this date. Date must be '
            'within the next year.'
        ),
        default=lambda: datetime.utcnow().date() + timedelta(days=31),
        validators=[validate_expires_at, validators.Optional()],
    )

    accept = SubmitField(_("Accept"), widget=Button(icon="fa fa-check"))
    reject = SubmitField(_("Reject"), widget=Button(icon="fa fa-times"))

    def validate_accept(form, field):
        """Validate that accept have not been set."""
        if field.data and form.reject.data:
            raise validators.ValidationError(
                _("Both reject and accept cannot be set at the same time.")
            )

    def validate_reject(form, field):
        """Validate that accept have not been set."""
        if field.data and form.accept.data:
            raise validators.ValidationError(
                _("Both reject and accept cannot be set at the same time.")
            )

    def validate_message(form, field):
        """Validate message."""
        if form.reject.data and not field.data.strip():
            raise validators.ValidationError(
                _("You are required to provide message to the requester when"
                  " you reject a request.")
            )


class DeleteForm(Form):
    """Form used for delete buttons."""

    link = HiddenField()

    delete = SubmitField(_("Revoke"), widget=Button(icon="fa fa-trash-o"))



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Zenodo module for providing access request feature."""

import os

from setuptools import find_packages, setup

readme = open('README.rst').read()
history = open('CHANGES.rst').read()

tests_require = [
    'check-manifest>=0.35',
    'coverage>=4.0',
    'invenio-records-ui>=1.0.1',
    'isort>=4.3.4',
    'mock>=1.3.0',
    'pydocstyle>=2.0.0',
    'pytest-cache>=1.0',
    'pytest-cov>=2.5.1',
    'pytest-pep8>=1.0.6',
    'pytest>=3.7.0',
]

extras_require = {
    'docs': [
        'Sphinx>=1.5,<1.6',
    ],
    'tests': tests_require,
}

extras_require['all'] = []
for reqs in extras_require.values():
    extras_require['all'].extend(reqs)

setup_requires = [
    'Babel>=1.3',
    'pytest-runner>=2.6.2',
]

install_requires = [
    'blinker>=1.4',
    'Flask-BabelEx>=0.9.2',
    'Flask-Breadcrumbs>=0.4.0',
    'Flask>=1.0.2',
    'invenio-access>=1.0.0a11',
    'invenio-accounts>=1.0.0b1',
    'invenio-db>=1.0.0b3',
    'invenio-files-rest>=1.0.0a14',
    'invenio-formatter>=1.0.0a2',
    'invenio-i18n>=1.0.0b1',
    'invenio-mail>=1.0.0a5',
    'invenio-pidstore>=1.0.0b1',
    'invenio-records>=1.0.0b1',
    'itsdangerous>=1.1.0',
    'WTForms>=2.0',
]

packages = find_packages()


# Get the version string. Cannot be done with import!
g = {}
with open(os.path.join('zenodo_accessrequests', 'version.py'), 'rt') as fp:
    exec(fp.read(), g)
    version = g['__version__']

setup(
    name='zenodo-accessrequests',
    version=version,
    description=__doc__,
    long_description=readme + '\n\n' + history,
    keywords='invenio TODO',
    license='GPLv2',
    author='CERN',
    author_email='info@zenodo.org',
    url='https://github.com/zenodo/zenodo-accessrequests',
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'invenio_admin.views': [
            'accessrequest_adminview = '
            'zenodo_accessrequests.admin:accessrequest_adminview',
            'secretlinks_adminview = '
            'zenodo_accessrequests.admin:secretlinks_adminview',
        ],
        'invenio_base.apps': [
            'zenodo_accessrequests = '
            'zenodo_accessrequests:ZenodoAccessRequests',
        ],
        'invenio_base.blueprints': [
            'zenodo_accessrequests_requests = '
            'zenodo_accessrequests.views.requests:blueprint',
            'zenodo_accessrequests_settings = '
            'zenodo_accessrequests.views.settings:blueprint',
        ],
        'invenio_db.models': [
            'zenodo_accessrequests = zenodo_accessrequests.models',
        ],
        'invenio_i18n.translations': [
            'messages = zenodo_accessrequests'
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Development Status :: 3 - Alpha',
    ],
)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Button widget for WTForms."""

from __future__ import absolute_import, print_function

from wtforms.compat import text_type
from wtforms.widgets import HTMLString, html_params


class Button(object):
    """Simple HTML5 button widget with support for including an icon."""

    input_type = 'submit'

    def __init__(self, icon=None):
        """Initialize button."""
        self._icon = icon

    def __call__(self, field, **kwargs):
        """Render button."""
        kwargs.setdefault('id', field.id)
        kwargs.setdefault('type', self.input_type)
        kwargs.setdefault('value', "y")
        return HTMLString('<button %s>%s%s</button>' % (
            html_params(name=field.name, **kwargs),
            "<i class=\"%s\"></i> " % self._icon if self._icon else "",
            text_type(field.label.text)
        ))



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Test signal receivers."""

from __future__ import absolute_import, print_function

import pytest
from helpers import create_access_request
from invenio_pidstore.errors import PIDDoesNotExistError
from mock import patch

from zenodo_accessrequests.models import AccessRequest
from zenodo_accessrequests.receivers import _send_notification, \
    create_secret_link


def test_send_notification(app, db):
    """Test sending of notifications."""
    with app.test_request_context():
        with patch('invenio_mail.tasks.send_email.delay') as mock:
            _send_notification(
                "info@invenio-software.org",
                "Test subject",
                "zenodo_accessrequests/emails/accepted.tpl",
                var1="value1",
            )
            assert mock.called
            msg = mock.call_args[0][0]
            assert msg['recipients'] == ["info@invenio-software.org"]
            assert msg['sender'] == 'info@zenodo.org'
            assert msg['subject'] == 'Test subject'


def test_create_secret_link(app, db, users, record_example,
                            access_request_confirmed):
    """Test creation of secret link."""
    with app.test_request_context():
        r = AccessRequest.query.filter_by(id=access_request_confirmed).first()
        create_secret_link(r)

        assert r.link.title == "Registered"
        assert "anotheremail@example.org" in r.link.description
        assert "Another Name" in r.link.description
        assert "Bla bla bla" in r.link.description


def test_create_secret_link_norecord(app, db, users):
    """Test creation of secret link with no record."""
    # Patch get_record
    with app.test_request_context():
        with patch('zenodo_accessrequests.utils.get_record',
                   return_value=None):
            with pytest.raises(PIDDoesNotExistError):
                r = create_access_request("1", users, confirmed=True)



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Base test case for access requests."""

from __future__ import absolute_import, print_function

from flask import current_app

from zenodo_accessrequests.models import AccessRequest


def create_access_request(pid_value, users, confirmed):
    """Access Request."""
    datastore = current_app.extensions['security'].datastore
    receiver = datastore.get_user(users['receiver']['id'])
    sender = datastore.get_user(users['sender']['id'])
    return AccessRequest.create(
        recid=pid_value,
        receiver=receiver,
        sender_full_name="Another Name",
        sender_email="anotheremail@example.org",
        sender=sender if confirmed else None,
        justification="Bla bla bla",
    )



# -*- coding: utf-8 -*-
#
# This file is part of Zenodo.
# Copyright (C) 2015, 2016 CERN.
#
# Zenodo is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Zenodo is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Zenodo. If not, see <http://www.gnu.org/licenses/>.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Model tests."""

from __future__ import absolute_import, print_function

from datetime import datetime, timedelta

import pytest
from flask import current_app
from helpers import create_access_request
from mock import Mock

from zenodo_accessrequests.errors import InvalidRequestStateError
from zenodo_accessrequests.models import AccessRequest, RequestStatus, \
    SecretLink
from zenodo_accessrequests.signals import link_created, link_revoked, \
    request_accepted, request_confirmed, request_created, request_rejected


def test_create_nouser(app, db, users, record_example):
    """Test access request creation without user."""
    mock_request_created = Mock()
    mock_request_confirmed = Mock()
    pid_value, record = record_example

    with app.test_request_context():
        with request_created.connected_to(mock_request_created):
            with request_confirmed.connected_to(mock_request_confirmed):
                datastore = current_app.extensions['security'].datastore
                receiver = datastore.get_user(users['receiver']['id'])

                r = AccessRequest.create(
                    recid=pid_value,
                    receiver=receiver,
                    sender_full_name="John Smith",
                    sender_email="info@invenio-software.org",
                    justification="Bla bla bla",
                )

                assert r.status == RequestStatus.EMAIL_VALIDATION
                assert r.sender_user_id is None
                assert r.created is not None
                assert r.modified is not None
                assert r.message == u''

                assert mock_request_created.called
                assert not mock_request_confirmed.called


def test_create_withuser(app, db, users, record_example):
    """Test access request creation with user."""
    mock_request_created = Mock()
    mock_request_confirmed = Mock()
    pid_value, record = record_example

    with app.test_request_context():
        with request_created.connected_to(mock_request_created):
            with request_confirmed.connected_to(mock_request_confirmed):
                datastore = current_app.extensions['security'].datastore
                receiver = datastore.get_user(users['receiver']['id'])
                sender = datastore.get_user(users['sender']['id'])

                r = AccessRequest.create(
                    recid=pid_value,
                    receiver=receiver,
                    sender_full_name="Another Name",
                    sender_email="anotheremail@example.org",
                    sender=sender,
                    justification="Bla bla bla",
                )

                assert r.status == RequestStatus.PENDING
                assert r.sender_user_id == sender.id
                assert r.created is not None
                assert r.modified is not None
                assert r.message == u''
                assert r.sender_full_name == u'Another Name'
                assert r.sender_email == u'anotheremail@example.org'

                assert not mock_request_created.called
                assert mock_request_confirmed.called


def test_accept(app, db, users, record_example, access_request_not_confirmed,
                access_request_confirmed):
    """Test accept signal and state."""
    mock_request_accepted = Mock()

    with app.test_request_context():
        with request_accepted.connected_to(mock_request_accepted):

            # Invalid operation when email not confirmed
            r = AccessRequest.query.filter_by(
                id=access_request_not_confirmed).first()
            with pytest.raises(InvalidRequestStateError):
                r.accept()
            assert not mock_request_accepted.called

            # Test signal and state
            r = AccessRequest.query.filter_by(
                id=access_request_confirmed).first()
            r.accept()
            assert r.status == RequestStatus.ACCEPTED
            assert mock_request_accepted.called
            assert mock_request_accepted.call_args[0][0] == r

            # Invalid operation for accepted request
            with pytest.raises(InvalidRequestStateError):
                r.confirm_email()


def test_reject(app, db, users, record_example, access_request_not_confirmed,
                access_request_confirmed):
    """Test reject signal and state."""
    mock_request_rejected = Mock()

    with app.test_request_context():
        with request_rejected.connected_to(mock_request_rejected):

            r = AccessRequest.query.filter_by(
                id=access_request_not_confirmed).first()
            with pytest.raises(InvalidRequestStateError):
                r.reject()
            assert not mock_request_rejected.called

            r = AccessRequest.query.filter_by(
                id=access_request_confirmed).first()
            r.reject()
            assert r.status == RequestStatus.REJECTED
            assert mock_request_rejected.call_args[0][0] == r

            with pytest.raises(InvalidRequestStateError):
                r.confirm_email()


def test_confirm_email(app, db, users, record_example,
                       access_request_not_confirmed):
    """Test confirm email signal and state."""
    mock_request_confirmed = Mock()
    pid_value, record = record_example

    with app.test_request_context():
        with request_confirmed.connected_to(mock_request_confirmed):

            r = AccessRequest.query.filter_by(
                id=access_request_not_confirmed).first()
            r.confirm_email()
            assert r.status == RequestStatus.PENDING
            assert mock_request_confirmed.call_args[0][0] == r


def test_query_by_receiver(app, db, users, record_example):
    """Test query by receiver."""
    pid_value, record = record_example

    with app.test_request_context():
        datastore = current_app.extensions['security'].datastore
        receiver = datastore.get_user(users['receiver']['id'])
        sender = datastore.get_user(users['sender']['id'])

        assert AccessRequest.query_by_receiver(receiver).count() == 0
        assert AccessRequest.query_by_receiver(sender).count() == 0

        # Create an accesrequest
        r = create_access_request(pid_value=pid_value, users=users,
                                  confirmed=False)

        assert AccessRequest.query_by_receiver(receiver).count() == 1
        assert AccessRequest.query_by_receiver(sender).count() == 0

        assert AccessRequest.get_by_receiver(r.id, receiver) is not None
        assert AccessRequest.get_by_receiver(r.id, sender) is None


def test_create_secret_link(app, db, users, record_example):
    """Test creation of secret link via token."""
    pid_value, record = record_example

    with app.test_request_context():
        datastore = current_app.extensions['security'].datastore
        receiver = datastore.get_user(users['receiver']['id'])

        r = create_access_request(pid_value=pid_value, users=users,
                                  confirmed=False)
        l = r.create_secret_link(
            "My link", "Link description",
            expires_at=datetime.utcnow() + timedelta(days=1)
        )
        assert r.link == l
        assert l.title == "My link"
        assert l.description == "Link description"
        assert l.extra_data == dict(recid=pid_value)
        assert l.owner == receiver


def test_creation(app, db, users, record_example):
    """Test link creation."""
    mock_link_created = Mock()
    pid_value, record = record_example

    with app.test_request_context():
        with link_created.connected_to(mock_link_created):
            datastore = current_app.extensions['security'].datastore
            receiver = datastore.get_user(users['receiver']['id'])

            l = SecretLink.create("Test title", receiver,
                                  dict(recid=pid_value),
                                  description="Test description")

            assert l.title == "Test title"
            assert l.description == "Test description"
            assert l.expires_at is None
            assert l.token != ''
            assert mock_link_created.called
            db.session.commit()

            l = SecretLink.query.get(l.id)
            assert SecretLink.validate_token(l.token, dict(recid=pid_value),)
            assert not SecretLink.validate_token(l.token, dict(recid='-1'))


def test_revoked(app, db, users):
    """Test link revocation."""
    mock_link_revoked = Mock()

    with app.test_request_context():
        with link_revoked.connected_to(mock_link_revoked):
            datastore = current_app.extensions['security'].datastore
            receiver = datastore.get_user(users['receiver']['id'])

            l = SecretLink.create(
                "Test title", receiver, dict(recid='123456'),
                description="Test description"
            )
            assert not l.is_revoked()
            assert l.is_valid()
            l.revoke()
            assert l.is_revoked()
            assert not l.is_valid()

            assert mock_link_revoked.called
            assert l.revoke() is False


def test_expired(app, db, users):
    """Test link expiry date."""
    with app.test_request_context():
        datastore = current_app.extensions['security'].datastore
        receiver = datastore.get_user(users['receiver']['id'])

        l = SecretLink.create(
            "Test title", receiver, dict(recid='123456'),
            description="Test description",
            expires_at=datetime.utcnow() - timedelta(days=1))
        assert l.is_expired()
        assert not l.is_valid()

        l = SecretLink.create(
            "Test title", receiver, dict(recid='123456'),
            description="Test description",
            expires_at=datetime.utcnow() + timedelta(days=1))
        assert not l.is_expired()
        assert l.is_valid()


def test_query_by_owner(app, db, users):
    """Test query by owner."""
    with app.test_request_context():
        datastore = current_app.extensions['security'].datastore
        receiver = datastore.get_user(users['receiver']['id'])
        sender = datastore.get_user(users['sender']['id'])

        assert SecretLink.query_by_owner(receiver).count() == 0
        assert SecretLink.query_by_owner(sender).count() == 0

        SecretLink.create("Testing", receiver, dict(recid='1'))

        assert SecretLink.query_by_owner(receiver).count() == 1
        assert SecretLink.query_by_owner(sender).count() == 0


def test_get_absolute_url(app, db, users):
    """Test absolute url."""
    with app.test_request_context():
        datastore = current_app.extensions['security'].datastore
        receiver = datastore.get_user(users['receiver']['id'])

        l = SecretLink.create("Testing", receiver, dict(recid='1'))
        url = l.get_absolute_url('invenio_records_ui.recid')
        assert "/records/1?" in url
        assert "token={0}".format(l.token) in url
